/*
	Copyright (C) 2011 Micha Hersch, University of Lausanne, Swiss Institute of Bioinformatics

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*!
 *	\file nestHistory.cpp
 *	\author Micha Hersch
 *	\date
 */
#include "nestHistory.h"
#include <iostream>

#define WITH_NANS

using namespace std;

NestHistory::NestHistory(){
  present=0;
}

void NestHistory::setSnake(const Snake *s,uint i, uint time){
  while(buffer.size()<=i){
    SnakeHistory sh;
    buffer.push_back(sh);
  }
  while(buffer[i].size()<=time){
    SimpleSnake ss;
    buffer[i].push_back(ss);
  }
  buffer[i][time].clear();
  for(u8 j=0;j<s->getSize();j++){
    buffer[i][time].push_back(s->getPoint(j));
  }
  //  cout<<"recording time "<<time<<endl;
  if (present<time){present=time;}
}


void NestHistory::getSnake(uint i, uint time,Snake& s){
  if(i>=buffer.size()){
    cerr<<"unvalid snake index"<<endl;
    return;
  }
  if(time>=buffer[i].size()){
    cerr<<"unvalid time index"<<endl;
    return;
  }
  s.empty();
  for(u8 j=0;j<buffer[i][time].size();j++){
    s.addPoint(buffer[i][time][j]);
  }
  return;
}
int NestHistory::getSnakeSize(uint i,uint tim)const{
  if(i>=buffer.size())return 0;
  if(tim>=buffer[i].size())return 0;
  return buffer[i][tim].size();
}

void NestHistory::printAllLengths(ostream& out){
  Snake s;
  for(uint t=0;t<=present;t++){
    for(uint i=0;i<buffer.size();i++){
      if(buffer[i].size()>t){
        getSnake(i,t,s);
	float len = s.getLength();
	out<<len<<" ";
      }
#ifdef WITH_NANS
      else{
	out<<"nan ";
      }
#endif
    }
    out<<endl;
  }
}
void NestHistory::printAllOrientations(ostream& out,float len){
  Snake s;
  for(uint t=0;t<=present;t++){
    for(uint i=0;i<buffer.size();i++){
      if(buffer[i].size()>t){
        getSnake(i,t,s);
	float ori = s.getOrientation(len);
	out<<ori<<" ";
      }
#ifdef WITH_NANS
      else{
	out<<"nan ";
      }
#endif
    }
    out<<endl;
  }
}
void NestHistory::printAllSegments(ostream& out){
 Snake s;
  for(uint t=0;t<=present;t++){
    for(uint i=0;i<buffer.size();i++){
      if(buffer[i].size()>t){
	//  getSnake(i,t,s);
	int s = buffer[i][t].size();
	Vec2 bot = buffer[i][t][s-1];
	Vec2 top = buffer[i][t][0];
	Vec2 seg = top-bot;
	out<<seg.x<<" "<<seg.y<<" ";
      }
#ifdef WITH_NANS
      else{
	out<<"nan nan ";
      }
#endif
    }
    out<<endl;
  }

}

void NestHistory::printAllTips(ostream& out){
 Snake s;
  for(uint t=0;t<=present;t++){
    for(uint i=0;i<buffer.size();i++){
      if(buffer[i].size()>t){
	//  getSnake(i,t,s);
	Vec2 top = buffer[i][t][0];
	out<<top.x<<" "<<top.y<<" ";
      }
#ifdef WITH_NANS
      else{
	out<<"nan nan ";
      }
#endif
    }
    out<<endl;
  }

}
